#!/bin/sh
# Hardware maintenance agent installer for VMware.
# The syslog message detection is invalidated.

MIACAT_VERSION="46-24"

MODEL_DATA_FILE="model_data.txt"
MIACAT_ROOT="/opt/hitachi/miacat"
MIACAT_PROG="${MIACAT_ROOT}/Program"
MAINTEDATA_PATH="${MIACAT_ROOT}/MainteData"
MODEL_CODE=""
MODEL_GROUP_CODE=""
MIACAT_CONF=${MIACAT_ROOT}/MainteData/smal2.conf

export W_PATH=`echo "${0}" | sed 's/\/[^/]*$//'`

set_conffile()
{
	local SEC="$1"
	local KEY="$2"
	local VAL="$3"
	local FILE="$4"
	local TEMP_FILE=${FILE}.tmp

	grep -q "^\\[${SEC}\\]" ${FILE}
	if [ $? -eq 0 ]; then
		sed -n "/^\\[${SEC}\\]/,/^\\[/p" ${FILE} | grep -q "^${KEY}="
		if [ $? -eq 0 ]; then
			sed "/^\\[${SEC}\\]/,/^\\[/s/^${KEY}=.*$/${KEY}=${VAL//\//\\/}/" ${FILE} > ${TEMP_FILE}
		else
			sed "/^\\[${SEC}\\]/a ${KEY}=${VAL}" ${FILE} > ${TEMP_FILE}
		fi
		[ $? -eq 0 ] && mv -f ${TEMP_FILE} ${FILE}
		rm -f ${TEMP_FILE}
	else
		echo "[${SEC}]" >> ${FILE}
		echo "${KEY}=${VAL}" >> ${FILE}
	fi
}

func_cancel() {
	echo ""
	echo "The installation was canceled."
	exit 1
}

func_group_sel() {
	SEL_NO_B=1
	for MODEL_GROUP in `cat ${W_PATH}/${MODEL_DATA_FILE} | sed -e "/^#/d" -e "/^$/d" | cut -f1 | sort | uniq`; do
		MODEL_GROUP_LIST[${SEL_NO_B}]="${MODEL_GROUP}"
		SEL_NO_B=$(($SEL_NO_B + 1))
	done

	if [ $SEL_NO_B -eq 2 ]; then
		MODEL_GROUP_CODE="${MODEL_GROUP_LIST[1]}"
	else
		while [ 1 ]; do
			echo ""
			echo "Select the model group."
			SEL_NO_L=1
			while [ $SEL_NO_L -lt $SEL_NO_B ]; do
				DISP[0]=""
				DISP[1]=""
				DISP[2]=""
				[[ ${MODEL_GROUP_LIST[${SEL_NO_L}]} ]] && DISP[0]=`printf "%3d: %s" $SEL_NO_L "${MODEL_GROUP_LIST[${SEL_NO_L}]}"`
				SEL_NO_L=$(($SEL_NO_L + 1))
				[[ ${MODEL_GROUP_LIST[${SEL_NO_L}]} ]] && DISP[1]=`printf "%3d: %s" $SEL_NO_L "${MODEL_GROUP_LIST[${SEL_NO_L}]}"`
				SEL_NO_L=$(($SEL_NO_L + 1))
				[[ ${MODEL_GROUP_LIST[${SEL_NO_L}]} ]] && DISP[2]=`printf "%3d: %s" $SEL_NO_L "${MODEL_GROUP_LIST[${SEL_NO_L}]}"`
				SEL_NO_L=$(($SEL_NO_L + 1))
				printf "  %-25s %-25s %-25s\n" "${DISP[0]}" "${DISP[1]}" "${DISP[2]}"
			done

			echo ""
			echo -n "Number(c:Cancel): "
			read SEL_NO
			if [ "${SEL_NO}" = "c" -o "${SEL_NO}" = "C" ]; then
				func_cancel
			fi
			if [ -n "`echo $SEL_NO | sed -e 's/[0-9]//g'`" -o -z "$SEL_NO" ]; then
				echo "Invalid number"
				echo ""
			else
				if [ $SEL_NO -ge $SEL_NO_B -o $SEL_NO -le 0 ]; then
					echo "Invalid number"
					echo ""
				else
					MODEL_GROUP_CODE="${MODEL_GROUP_LIST[${SEL_NO}]}"
					echo "HA8000/${MODEL_GROUP_CODE} model was selected."
					break
				fi
			fi
		done
	fi
}

func_model_sel() {
	echo ""
	echo "Select model"
	SEL_NO_B=1
	for MODEL_CODE in `cat ${W_PATH}/${MODEL_DATA_FILE} | tr -s "\t" | grep "^${MODEL_GROUP_CODE}	" | cut -f2 `; do
		MODEL_CODE_LIST[${SEL_NO_B}]="${MODEL_CODE}"
		SEL_NO_B=$(($SEL_NO_B + 1))
	done
	SEL_NO_B=1
	for MODEL_CODE in `cat ${W_PATH}/${MODEL_DATA_FILE} | tr -s "\t" | grep "^${MODEL_GROUP_CODE}	" | tr " " "_" | cut -f3 `; do
		MODEL_NAME=`echo "${MODEL_CODE}" | tr "_" " "`
		MODEL_NAME_LIST[${SEL_NO_B}]="${MODEL_NAME}"
		SEL_NO_B=$(($SEL_NO_B + 1))
	done


	while [ 1 ]; do
		SEL_NO_L=1
		while [ $SEL_NO_L -lt $SEL_NO_B ]; do
			DISP[0]=""
			DISP[1]=""
			DISP[2]=""
			[[ ${MODEL_NAME_LIST[${SEL_NO_L}]} ]] && DISP[0]=`printf "%3d: %s" $SEL_NO_L "${MODEL_NAME_LIST[${SEL_NO_L}]}"`
			SEL_NO_L=$(($SEL_NO_L + 1))
			[[ ${MODEL_NAME_LIST[${SEL_NO_L}]} ]] && DISP[1]=`printf "%3d: %s" $SEL_NO_L "${MODEL_NAME_LIST[${SEL_NO_L}]}"`
			SEL_NO_L=$(($SEL_NO_L + 1))
			[[ ${MODEL_NAME_LIST[${SEL_NO_L}]} ]] && DISP[2]=`printf "%3d: %s" $SEL_NO_L "${MODEL_NAME_LIST[${SEL_NO_L}]}"`
			SEL_NO_L=$(($SEL_NO_L + 1))
			printf "  %-25s %-25s %-25s\n" "${DISP[0]}" "${DISP[1]}" "${DISP[2]}"
		done

		echo -n "Number(c:Cancel): "
		read SEL_NO
		if [ "${SEL_NO}" = "c" -o "${SEL_NO}" = "C" ]; then
			func_cancel
		fi
		if [ -n "`echo $SEL_NO | sed -e 's/[0-9]//g'`" -o -z "$SEL_NO" ]; then
			echo "Invalid number"
			echo ""
		else
			if [ $SEL_NO -ge $SEL_NO_B -o $SEL_NO -le 0 ]; then
				echo "Invalid number"
				echo ""
			else
				MODEL_CODE="${MODEL_CODE_LIST[${SEL_NO}]}"
				echo "HA8000/${MODEL_NAME_LIST[${SEL_NO}]} model was selected."
				break
			fi
		fi
	done
}

##################
# VMware function
IPMIRAWCMD=/opt/hitachi/tools/miacat/ipmiRawCmd
IPMI_RES=
func_IPMI_ReadFRU()
{
	local FRU_OFFSET=$1
	local FRU_SIZE=$2
	local CMDARG
	local FLDS
	
	CMDARG=`printf "0x%02X 0x%02X 0x%02X" $((${FRU_OFFSET} & 0xFF)) $((${FRU_OFFSET}>>8)) ${FRU_SIZE}`
	#IPMI_RES=`${IPMIRAWCMD} raw 0x0A 0x11 0x00 ${CMDARG} 2> /dev/null | tr -d "\r" | tr -d "\n"`
	IPMI_RES=`${IPMIRAWCMD} raw 0x0A 0x11 0x00 ${CMDARG}`
	if [ $? -ne 0 ]; then
		echo "ERROR: ipmiRawCmd command failed."
		exit 1
	fi
	IPMI_RES=`echo "${IPMI_RES}" | tr -d "\r" | tr -d "\n"`
	if [ "${IPMI_RES:0:2}" ==  "00" ]; then
		echo "ERROR: ipmiRawCmd command failed. (No Data)"
		exit 1
	fi
	return 0
}

func_vmware_get_fru()
{
	local FLDS
	
	if [ ! -x ${IPMIRAWCMD} ]; then
		echo "ERROR: ipmiRawCmd command not found."
		exit 1
	fi
	
	IFS=' '
	
	func_IPMI_ReadFRU 0x04 0x01
	FLDS=(${IPMI_RES})
	if [ "${FLDS[0]}" != "01" ]; then
		echo "ERROR: ipmiRawCmd command failed. (Read error)"
		exit 1
	fi
	FRU_PIA=$((0x${FLDS[1]}*8))
	# Manufacturer Name
	FRU_POS=$((${FRU_PIA} + 3))
	func_IPMI_ReadFRU ${FRU_POS} 0x01
	FLDS=(${IPMI_RES})
	FRU_POS=$((${FRU_POS} + 1 + 0x${FLDS[1]:1}))
	# Product Name
	func_IPMI_ReadFRU ${FRU_POS} 0x01
	FLDS=(${IPMI_RES})
	FRU_POS=$((${FRU_POS} + 1 + 0x${FLDS[1]:1}))
	# Product Part/Model Number
	func_IPMI_ReadFRU ${FRU_POS} 0x01
	FLDS=(${IPMI_RES})
	FRU_POS=$((${FRU_POS} + 1 + 0x${FLDS[1]:1}))
	# Product Version
	func_IPMI_ReadFRU ${FRU_POS} 0x01
	FLDS=(${IPMI_RES})
	if [ $((0x${FLDS[1]:1})) -ne 10 ]; then
		echo "ERROR: Failed to read FRU."
		exit 1
	fi
	func_IPMI_ReadFRU $((${FRU_POS}+1)) 0x10
	FLDS=(${IPMI_RES})
	if [ $((${FLDS[0]})) -eq 0 ]; then
		echo "ERROR: ipmiRawCmd command failed. (No Data)"
		exit 1
	fi
	MODEL_CODE=`printf "\x${FLDS[1]}\x${FLDS[2]}\x${FLDS[3]}\x${FLDS[4]}\x${FLDS[5]}\x${FLDS[6]}\x${FLDS[7]}\x${FLDS[8]}\x${FLDS[9]}\x${FLDS[10]}"`
	return 0
}

echo "----------------------------------------------------------------"
#echo " Hardware maintenance agent is installed in guest OS of VMware. "
echo " log monitor Ver. ${MIACAT_VERSION} Installer"
echo " for Guest OS of VMware"
echo "----------------------------------------------------------------"
# Run user check
if [ `id -u` != "0" ]; then
	echo "Please execute it by \"root\" user."
	exit 1
fi

# Analyze the arguments.
MODEL_SELECT=MENU
if [ "$1" != "-m" ]; then
	if [ -f /etc/vma-release ]; then
		MODEL_SELECT=AUTO_VMWARE
	fi
fi

# Have you already installed it?
UPDATE_MODE=0
MIACAT_INSTALLED=`rpm -q MIACAT 2> /dev/null | grep "^MIACAT-"`
if [ -n "${MIACAT_INSTALLED}" ]; then
	echo "It has already installed it."
	while [ 1 ]; do
		echo -n "Do you update it? [y/n]: "
		read ANS
		if [ "${ANS}" = "y" -o "${ANS}" = "Y" ]; then
			UPDATE_MODE=1
			break
		fi
		if [ "${ANS}" = "n" -o "${ANS}" = "N" ]; then
			func_cancel
		fi
	done
fi

if [ $UPDATE_MODE -eq 1 ]; then
	rpm -Uvh ${W_PATH}/MIACAT-*.i386.rpm
	RPM_RET=$?
	SUCCESSFULLY_MSG="Update"
else
	if [ "${MODEL_SELECT}" == "AUTO_VMWARE" ]; then
		func_vmware_get_fru
	else
		func_group_sel
		func_model_sel
	fi
	export MIACAT_DEBUG_INSTALL=${MODEL_CODE}
	rpm -ivh ${W_PATH}/MIACAT-*.i386.rpm
	RPM_RET=$?
	SUCCESSFULLY_MSG="Installation"
fi
echo ""
RET_CD=0
if [ $RPM_RET -eq 0 ]; then
	### The syslog message detection is invalidated.
	##LOGANALYZE_PATH=`cat ${MAINTEDATA_PATH}/ast_setup.conf | grep "^OSMSGWATCH_DIR=" | cut -d= -f2`
	##SRC_NUMBER=`cat ${MAINTEDATA_PATH}/LogAnalyze/${LOGANALYZE_PATH}/PCIAnalyze.tbl | grep SMAL2_MainteAgtSvc | cut -d= -f1 | cut -d_ -f2`
	##if [ -n "${SRC_NUMBER}" ]; then
	##	cat ${MAINTEDATA_PATH}/LogAnalyze/${LOGANALYZE_PATH}/PCIAnalyze.tbl | grep -E "^([^S][^R][^C][^_]|SRC_${SRC_NUMBER}=|^$)" > ${MAINTEDATA_PATH}/LogAnalyze/${LOGANALYZE_PATH}/PCIAnalyze.tbl_tmp
	##	mv -f ${MAINTEDATA_PATH}/LogAnalyze/${LOGANALYZE_PATH}/PCIAnalyze.tbl_tmp ${MAINTEDATA_PATH}/LogAnalyze/${LOGANALYZE_PATH}/PCIAnalyze.tbl
	##	chmod 644 ${MAINTEDATA_PATH}/LogAnalyze/${LOGANALYZE_PATH}/PCIAnalyze.tbl
	##	echo "${SUCCESSFULLY_MSG} successfully."
	##	RET_CD=0
	##else
	##	echo "${SUCCESSFULLY_MSG} failure."
	##	RET_CD=1
	##fi
	#set_conffile "SOFTWARE\\HITACHI\\miacat\\Maintenance Agent Service\\LogWatch\\EventLog" AddOEMSELtoBMC 0 ${MIACAT_CONF}
	for MSGFILE in ${MAINTEDATA_PATH}/LogAnalyze/*.tbl
	do
		if echo "${MSGFILE}" | grep -q "RF0_MIACAT.tbl"; then
			continue
		fi
		if echo "${MSGFILE}" | grep -q "R62_BR1200.tbl"; then
			continue
		fi
		if echo "${MSGFILE}" | grep -q "R63_BR1200.tbl"; then
			continue
		fi
		rm -f ${MSGFILE}
	done
	
	echo "${SUCCESSFULLY_MSG} successfully."
	RET_CD=0
else
	echo "${SUCCESSFULLY_MSG} failure."
	RET_CD=1
fi
exit $RET_CD
